@@ -49,9 +49,10 @@ import signal
 import threading
 from collections import namedtuple
 from types import FrameType
-from typing import Any, Callable, List, Optional, Set, Tuple
+from typing import Any, Callable, Dict, List, Optional, Set, Tuple
 
 import pika
+import pika.exceptions
 from pika.exchange_type import ExchangeType
 
 from yapw.decorators import default_decode, halt
@@ -75,7 +76,18 @@ def _on_message(
     (threads, decorator, decode, callback, state) = args
     thread = threading.Thread(target=decorator, args=(decode, callback, state, channel, method, properties, body))
     thread.start()
-    threads.append(thread)
+
+    alive = [thread]
+    # Clean up threads occasionally to avoid a memory leak.
+    if len(threads) > 100:
+        for t in threads:
+            if t.is_alive():
+                alive.append(t)
+            # Python 3.8 and below has a memory leak. https://github.com/python/cpython/pull/26103
+            else:
+                t.join()
+
+    threads[:] = alive
 
 
 class Base:
@@ -211,7 +223,9 @@ class Publisher:
         if self.exchange:
             self.channel.exchange_declare(exchange=self.exchange, exchange_type=exchange_type, durable=self.durable)
 
-    def declare_queue(self, queue: str, routing_keys: Optional[List[str]] = None) -> None:
+    def declare_queue(
+        self, queue: str, routing_keys: Optional[List[str]] = None, arguments: Optional[Dict[str, str]] = None
+    ) -> None:
         """
         Declare a queue, and bind it to the exchange with the routing keys. If no routing keys are provided, the queue
         is bound to the exchange using its name as the routing key.
@@ -223,7 +237,7 @@ class Publisher:
             routing_keys = [queue]
 
         formatted = self.format_routing_key(queue)
-        self.channel.queue_declare(queue=formatted, durable=self.durable)
+        self.channel.queue_declare(queue=formatted, durable=self.durable, arguments=arguments)
 
         for routing_key in routing_keys:
             routing_key = self.format_routing_key(routing_key)
@@ -268,7 +282,7 @@ class Threaded:
 
     # Attributes that this mixin expects from base classes.
     format_routing_key: Callable[["Threaded", str], str]
-    declare_queue: Callable[["Threaded", str, Optional[List[str]]], None]
+    declare_queue: Callable[["Threaded", str, Optional[List[str]], Optional[Dict[str, str]]], None]
     connection: pika.BlockingConnection
     channel: pika.adapters.blocking_connection.BlockingChannel
 
@@ -295,6 +309,7 @@ class Threaded:
         queue: str,
         routing_keys: Optional[List[str]] = None,
         decorator: Decorator = halt,
+        arguments: Optional[Dict[str, str]] = None,
     ) -> None:
         """
         Declare a queue, bind it to the exchange with the routing keys, and start consuming messages from that queue.
@@ -308,8 +323,9 @@ class Threaded:
         :param queue: the queue's name
         :param routing_keys: the queue's routing keys
         :param decorator: the decorator of the consumer callback
+        :param arguments: the ``arguments`` parameter to the ``queue_declare`` method
         """
-        self.declare_queue(queue, routing_keys)
+        self.declare_queue(queue, routing_keys, arguments)
         formatted = self.format_routing_key(queue)
 
         # Don't pass `self` to the callback, to prevent use of unsafe attributes and mutation of safe attributes.
@@ -323,7 +339,11 @@ class Threaded:
         logger.debug("Consuming messages on channel %s from queue %s", self.channel.channel_number, formatted)
         try:
             self.channel.start_consuming()
-        finally:
+        except pika.exceptions.ConnectionClosedByBroker:
+            for thread in threads:
+                thread.join()
+            # The connection is already closed.
+        else:
             for thread in threads:
                 thread.join()
             self.connection.close()
